/*
 * Copyright (c) 2001-2004 Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 *
 * Author: Huawei Technologies
 *
 */

#ifndef __LWIPSOCK_H__
#define __LWIPSOCK_H__

#include "lwip/opt.h"
#include "lwip/api.h"

#include "eventpoll.h"

/* move some definitions to the lwipsock.h for libnet to use, and
 * at the same time avoid conflict between lwip/sockets.h and sys/socket.h
 */

/* --------------------------------------------------
 * the following definition is copied from lwip/priv/tcpip_priv.h
 * --------------------------------------------------
 */

/** This is overridable for the rare case where more than 255 threads
 * select on the same socket...
 */
#ifndef SELWAIT_T
#define SELWAIT_T u8_t
#endif

union lwip_sock_lastdata {
  struct netbuf *netbuf;
  struct pbuf *pbuf;
};

#if USE_LIBOS
struct protocol_stack;
struct wakeup_poll;
struct rte_ring;
#include <rte_common.h>
#endif
/** Contains all internal pointers and states used for a socket */
struct lwip_sock {
  /** sockets currently are built on netconns, each socket has one netconn */
  struct netconn *conn;
  /** data that was left from the previous read */
  union lwip_sock_lastdata lastdata;
#if LWIP_SOCKET_SELECT || LWIP_SOCKET_POLL
  /** number of times data was received, set by event_callback(),
      tested by the receive and select functions */
  s16_t rcvevent;
  /** number of times data was ACKed (free send buffer), set by event_callback(),
      tested by select */
  u16_t sendevent;
  /** error happened for this socket, set by event_callback(), tested by select */
  u16_t errevent;
  /** counter of how many threads are waiting for this socket using select */
  SELWAIT_T select_waiting;
#endif /* LWIP_SOCKET_SELECT || LWIP_SOCKET_POLL */
#if LWIP_NETCONN_FULLDUPLEX
  /* counter of how many threads are using a struct lwip_sock (not the 'int') */
  u8_t fd_used;
  /* status of pending close/delete actions */
  u8_t fd_free_pending;
#define LWIP_SOCK_FD_FREE_TCP  1
#define LWIP_SOCK_FD_FREE_FREE 2
#endif

#if USE_LIBOS
  char pad0 __rte_cache_aligned;
  /* app thread use */
  struct pbuf *recv_lastdata; /* unread data in one pbuf */
  uint16_t remain_len;
  uint32_t epoll_events; /* registered events, EPOLLONESHOT write frequently */
  volatile uint32_t events; /* available events */
  struct list_node event_list;

  char pad1 __rte_cache_aligned;
  /* app and stack thread all use */
  uint32_t in_send; /* avoid sock too much send rpc msg*/
  bool read_wait;
  char pad2 __rte_cache_aligned;
  /* stack thread all use */
  struct list_node recv_list;
  struct list_node send_list;
  struct pbuf *lwip_lastdata;
  struct pbuf *send_lastdata;
  struct pbuf *send_pre_del;

  char pad3 __rte_cache_aligned;
  /* nerver change */
  struct wakeup_poll *wakeup;
  epoll_data_t ep_data;
  struct lwip_sock *listen_next; /* listenfd list */
  struct protocol_stack *stack;
  struct rte_ring *recv_ring;
  struct rte_ring *send_ring;
#endif
};

#ifndef set_errno
#define set_errno(err) do { if (err) { errno = (err); } } while(0)
#endif


/* --------------------------------------------------
 * --------------- LIBNET references ----------------
 * --------------------------------------------------
 */
#if USE_LIBOS
extern uint32_t sockets_num;
extern struct lwip_sock *sockets;
extern void gazelle_connected_callback(struct netconn *conn);
extern void add_recv_list(int32_t fd);
extern ssize_t read_lwip_data(struct lwip_sock *sock, int32_t flags, u8_t apiflags);
extern struct pbuf *write_lwip_data(struct lwip_sock *sock, uint16_t remain_size, uint8_t *apiflags);
extern void gazelle_init_sock(int32_t fd);
extern void gazelle_clean_sock(int32_t fd);
extern void write_lwip_over(struct lwip_sock *sock);
#endif /* USE_LIBOS */

struct lwip_sock *get_socket(int s);
struct lwip_sock *get_socket_by_fd(int s);
void lwip_sock_init(void);
void lwip_exit(void);

#endif /* __LWIPSOCK_H__ */
